/****************************************************************************
                  INTERNATIONAL AVS CENTER
	(This disclaimer must remain at the top of all files)

WARRANTY DISCLAIMER

This module and the files associated with it are distributed free of charge.
It is placed in the public domain and permission is granted for anyone to use,
duplicate, modify, and redistribute it unless otherwise noted.  Some modules
may be copyrighted.  You agree to abide by the conditions also included in
the AVS Licensing Agreement, version 1.0, located in the main module
directory located at the International AVS Center ftp site and to include
the AVS Licensing Agreement when you distribute any files downloaded from 
that site.

The International AVS Center, MCNC, the AVS Consortium and the individual
submitting the module and files associated with said module provide absolutely
NO WARRANTY OF ANY KIND with respect to this software.  The entire risk as to
the quality and performance of this software is with the user.  IN NO EVENT
WILL The International AVS Center, MCNC, the AVS Consortium and the individual
submitting the module and files associated with said module BE LIABLE TO
ANYONE FOR ANY DAMAGES ARISING FROM THE USE OF THIS SOFTWARE, INCLUDING,
WITHOUT LIMITATION, DAMAGES RESULTING FROM LOST DATA OR LOST PROFITS, OR ANY
SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES.

This AVS module and associated files are public domain software unless
otherwise noted.  Permission is hereby granted to do whatever you like with
it, subject to the conditions that may exist in copyrighted materials. Should
you wish to make a contribution toward the improvement, modification, or
general performance of this module, please send us your comments:  why you
liked or disliked it, how you use it, and most important, how it helps your
work. We will receive your comments at avs@ncsc.org.

Please send AVS module bug reports to avs@ncsc.org.

******************************************************************************/
/* Module Name: "geom2wave" (Output) (Subroutine)                  */
/* Author: Steve A. Chall,NCSC,919-248-1191                              */
/* Date Created: Wed Feb 23 09:59:42 1994                                */
/*                                                                       */
/* input 0 "GeomIn" geom REQUIRED                                        */
/* param 0 "outFNameBase" typein "fromAVS" "" ":"                        */
/* 				                                         */
/* Write out all the appropriate AVS geometries in the current edit list */
/* as Wavefront object (.obj) files. Currently converts polyhedra and 	 */
/* polytriangle strips.  						 */


#include <stdio.h>
#include <time.h>
#include <avs/avs.h>
#include <avs/port.h>
#include <avs/geom.h>
#include <avs/geom_edit.h>

 
/*void ProcessEditList(GEOMedit *geom, char *fileNameBase);
int WritePolyhedron(GEOMpolyh *ph, char *fileNameBase, int currFNum);
int WritePolytri(GEOMobj *obj, char *fileNameBase, int currFNum);*/
void ProcessEditList();
int WritePolyhedron();
int WritePolytri();

 
/* *****************************************/
/*  Module Description                     */
/* *****************************************/
int geom2wave_desc()
{
	int in_port, out_port, param, iresult;
	extern int geom2wave_compute();

	AVSset_module_name("Geom to Wavefront", MODULE_RENDER);

	/* Input Port Specifications               */
	in_port = AVScreate_input_port("GeomIn", "geom", REQUIRED);

	/* Parameter Specifications                */
	param = AVSadd_parameter("outFNameBase", "string", "fromAVS", "", ":");
	AVSconnect_widget(param, "typein");

	AVSset_compute_proc(geom2wave_compute);
	return(1);
}
 

/* *****************************************/
/* Module Compute Routine                  */
/* *****************************************/
int geom2wave_compute(GeomIn, fileNameBase)
	GEOMedit_list GeomIn;
	char *fileNameBase;
{
        if ((GeomIn == NULL) || (GeomIn->l == NULL)) {
                fprintf(stderr, "geom2wave_compute:  null edit list.\n");
        }
        else {
                ProcessEditList(GeomIn->l, fileNameBase);
        }

        return(1);
}
 

/* ***********************************************************************/
/* Initialization for modules contained in this file.                    */
/* ***********************************************************************/
static int ((*mod_list[])()) = {
	geom2wave_desc
};
#define NMODS (sizeof(mod_list) / sizeof(char *))


AVSinit_modules()
{
	AVSinit_from_module_list(mod_list, NMODS);
}


/* ***********************************************************************/
/* Traverse the edit list of which geom is the first member, processing  */
/* each list element in turn.						 */
/* ***********************************************************************/
void ProcessEditList(geom, fNBase)
        GEOMedit *geom;
        char *fNBase;

{
        GEOMobj *obj = NULL;
        static int fNum = 0;         	/* object file number    	 */

        while (geom != NULL) {

                if (geom->type == GEOM_EDIT_GEOMETRY) {
                        obj = (GEOMobj *)(geom->data);

                        if (obj == NULL) {
                                fprintf(stderr, " geom2wave:  null GEOMobj.\n");
                                return;
                        }

                        switch (obj->type) {
                                case GEOM_POLYHEDRON:
                                        WritePolyhedron(&(obj->d.ph), fNBase,
						fNum++);
                                        break;
                                case GEOM_POLYTRI:
                                        WritePolytri(obj, fNBase, fNum++);
                                        break;
                                case GEOM_MESH:
                                case GEOM_SPHERE:
                                case GEOM_LABEL:
					/* not implemented */
                                        break;
                                default:
					fprintf(stderr,
					"geom2wave:  strange object type.\n");
                                        break;
                        };
                }

        	geom = geom->next;
        }

        return;
}


/* ***********************************************************************/
/* Write a Wavefront .obj file equivalent to the AVS GEOMpolyh pointed   */
/* to by parameter ph.							 */
/* ***********************************************************************/
int WritePolyhedron(ph, fNameBase, fNum)
        GEOMpolyh *ph;                  /* polyhedron                    */
	char *fNameBase;                /* (output) file name base       */
	int fNum;			/* number for the current file   */
{
        GEOMvert_list *vData = NULL;	/* pointer to vertex struct      */
        float *vPtr = NULL;             /* pointer to array of vertices  */
        int numVerts = 0;               /* total # of vertices           */
        GEOMp_list *pList = NULL;	/* pointer to polygon struct     */
        int *pPtr = NULL;               /* pointer to array of polygons  */
        int vNum;                       /* # vertices in curr. polygon   */
        int v;                          /* loop counter variable         */
        FILE *f = NULL;                 /* pointer to .obj file          */
	static char fName[256];         /* full output file name         */
	time_t t;                       /* used to write date to file    */

        vData = &(ph->verts);
        vPtr = vData->l;

        if (vPtr == NULL) {
                fprintf(stderr, "geom2wave:  null vertex list.\n");
                return(0);
        }

        numVerts = vData->n;

        pList = &(ph->ptlist);
        pPtr = pList->l;

        if (pPtr == NULL) {
                fprintf(stderr, "geom2wave:  null polygon list.\n");
                return(0);
        }

        sprintf(fName, "%s.%d.obj", fNameBase, fNum);

	if (!(f = fopen(fName,"w"))) {
                fprintf(stderr, "geom2wave:  failed to open file \"%s\"",
                        " for writing.\n", fName);
                return(0);
        }

	time(&t);
        fprintf(f,"# %s#\n#\n\ng\n", ctime(&t));

        for (v = 0; v < numVerts; v++) {                                                        fprintf(f, "v %f", *(vPtr++));
                fprintf(f, " %f", *(vPtr++));
                fprintf(f, " %f\n", *(vPtr++));
        }

	fprintf(f, "# %d vertices\n\n", numVerts);
        fprintf(f, "g %s.%d\n", fNameBase, fNum);

        vNum = *(pPtr++);

        while (vNum != 0) {
                fprintf(f, "f ");

                for (v = 1; v <= vNum; v++) {
                        fprintf(f, "%d ", *(pPtr++));
                }

                fprintf(f, "\n");
                vNum = *(pPtr++);
        }

        fprintf(f, "# %d elements\n", pList->nps);
        fclose(f);

	return(1);
}


/* ***********************************************************************/
/* Write a Wavefront .obj file equivalent to the AVS GEOMpolytri assumed */
/* to be in obj.							 */
/* ***********************************************************************/
int WritePolytri(obj, fNameBase, fNum)
	GEOMobj *obj;	
        char *fNameBase;                /* (output) file name base       */
        int fNum;                       /* number for the current file   */
{
        int i, j;                    	/* loop counter variables        */
        FILE *f = NULL;                 /* pointer to .obj file          */
        static char fName[256];         /* full output file name         */
        time_t t;                       /* used to write date to file    */

        sprintf(fName, "%s.%d.obj", fNameBase, fNum);

        if (!(f = fopen(fName,"w"))) {
                fprintf(stderr, "geom2wave:  failed to open file \"%s\"",
                        " for writing.\n", fName);
                return(0);
        }

        time(&t);
        fprintf(f,"# %s#\n#\n\ng\n", ctime(&t));

	/* The following nested loop structure is from Jeff Vroom's
	 * "AVS Technote:  Interpreting GEOM Information," Proceedings,
	 * 1st International AVS Users' Group Conference, RTP, NC, 1992:
	 */
        for (j = 0; j < PT(obj).npts; j++) {

                for (i = 0; i < PT(obj).ptverts[j].n; i++) {
                        fprintf(f, "v %f %f %f\n",
                                PT(obj).ptverts[j].l[3 * i],
                                PT(obj).ptverts[j].l[3 * i + 1],
                                PT(obj).ptverts[j].l[3 * i + 2]);
                }
        }

        if (obj->data & GEOM_NORMALS) {

        	for (j = 0; j < PT(obj).npts; j++) {

                	for (i = 0; i < PT(obj).ptverts[j].n; i++) {
                        	fprintf(f, "vn %f %f %f\n",
                                	PT(obj).ptnormals[j].l[3 * i],
                                	PT(obj).ptnormals[j].l[3 * i + 1],
                                	PT(obj).ptnormals[j].l[3 * i + 2]);
                	}
        	}
	}

        fprintf(f, "g %s.%d\n", fNameBase, fNum);

        for (j = 0; j < PT(obj).npts; j++) {

                for (i = 1; i <= (PT(obj).ptverts[j].n - 2); i++) {
                        fprintf(f, "f %d %d %d\n", ((i % 2) ? (i + 2) : i),
				i + 1, ((i % 2) ? i : (i + 2)));
                }
        }

        fclose(f);

	return(1);
}
